 title  "Switcher - Automatically switch up to four A/V signals."
;
;  This application uses the ADC capability of the 16F819 to check the status of up to four video 
;  signals and switch to the "active" signal with the highest priority.  Manual override buttons 
;  are also implemented.
;
;  Hardware Notes:
;   This application runs on a 16F819 executing at 4 MHz on the internal oscillator.
;
;  Mike Gardi
;  2004.02.18
;
  LIST R=DEC
  INCLUDE "p16f819.inc"

;
;  Registers
;
 CBLOCK 0x020
CountToQuarter					; Need to count 250 microseconds in ISR for 1/4 second.
QuarterFlag						; ISR lets Mainline know 1/4 seconds passed via this flag.
ButtonInput						; Used to check the status of the override buttons.
ButtonSelected					; Remember what buttons were pressed when we last selected.
VideoInput						; Set with video inputs that are live.
VideoSample						; Used to set the video inputs when sampling sources.
VideoSelected					; Remember what video sources were active when we last selected.
DebounceCount					; Timed 1/20 of a second delay used to debounce button presses.
HighestSample					; Largest result so far from the current ADC input.
VideoOn							; Each input has it's own values used to detect ON or OFF.
VideoOff 
Temp							; General register for use by Mainline.
 ENDC

;
;  Save W and STATUS registers to addresses that span pages.
;
#define	_w				0x70
#define _status			0x71

;
;  Pin Assignments.
;
#define VIDEO_IN_1		PORTA, 0	; Inputs used to sample the video signals.
#define VIDEO_IN_2      PORTA, 1
#define VIDEO_IN_3		PORTA, 2
#define VIDEO_IN_4      PORTA, 3

#define BUTTON_1		PORTB, 0	; Inputs used to read the 4 override buttons.
#define BUTTON_2		PORTB, 1
#define BUTTON_3		PORTB, 2
#define BUTTON_4		PORTB, 3

#define LED_1			PORTB, 4	; Outputs used to select the indicator LEDs.
#define LED_2			PORTB, 5
#define LED_3			PORTB, 6
#define LED_4			PORTB, 7
#define LED_5			PORTA, 6

#define A0				7			; PORTA pins used to select the input channel address.
#define A1 				4

;
;  Define the values that will be used to detect ON/OFF conditions.
;
#define VIDEO_1_OFF		8
#define VIDEO_2_OFF		3
#define VIDEO_3_OFF		3
#define VIDEO_4_OFF		4

#define VIDEO_1_ON		10
#define VIDEO_2_ON		10
#define VIDEO_3_ON		10
#define VIDEO_4_ON		10
						
;
;  Set Fuses.
;
 __CONFIG _CP_OFF & _CPD_OFF & _INTRC_IO & _PWRTE_ON  & _WDT_OFF & _LVP_OFF & _BODEN_OFF & _MCLR_OFF & _WRT_ENABLE_OFF

;
;  Initialize Switcher
;
  org    0
  goto   Mainline

;
;  Interrupt Handler
;
 org     4
  movwf  _w						; Save Context Registers.
  movf   STATUS, w
  movwf  _status

  bcf    INTCON, TMR0IF			; Reset the Interrupt Flag. Assume TMR0 is the only enabled Interrupt.
  movlw	 12						; Adjust TMR0 to get us back to 1000 microseconds.
  addwf  TMR0, f

  decf   DebounceCount, f		; Manage a debounce delay on behalf of the Mainline.

  decfsz CountToQuarter, f		; Has 1/4 of a second passed?
  goto	 IntEnd					;   No - Done interrupt.
  incf	 QuarterFlag, f			;  Yes - Let Mainline know that 1/4 second has passed.
  movlw	 250					; Setup for another 1/4 second.
  movwf	 CountToQuarter				

IntEnd
  movf   _status, w				; Restore the Context Registers.
  movwf  STATUS
  swapf  _w, f
  swapf  _w, w
  retfie

;
;  Switcher Mainline
;
Mainline	
  bcf    STATUS, RP0         	; Make sure in Bank 0.
  movlw  b'0000000'				; Disable capture/compare/PWM.
  movwf  CCP1CON					

  bsf    STATUS, RP0       		; Setup Bank 1 registers.

  movlw  0x051					; Setup TMR0 for a 1/1000 of a second delay.
  movwf  OPTION_REG				; Weak pullups enabled.

  movlw  b'00001111'			; B0-B3 Button Inputs  B4-B7 LED Outputs
  movwf  PORTB

  movlw  b'00000010'			; A0-A4 set for Analog input.  Left justified A/D result.
  movwf  ADCON1				

  movlw  b'00101111'			; A0-A3 Analog Video Inputs  A4,A7 A/V Address Outputs, A6 LED Output, A5 Input.
  movwf  PORTA

  movlw  b'01100000'			; Set internal oscillator to 4Mhz.
  movwf  OSCCON

  bcf    STATUS, RP0         	; Done setup Bank 1 registers.  Back to Bank 0.

  movlw  b'11110000'			; Video Indicator LEDs off.
  movwf  PORTB

  movlw  b'01000000'			; Override LED Off.  A/V switched by default to source 1.
  movwf  PORTA

  movlw	 0						
  movwf  QuarterFlag			; Initialize the quarter second indicator.
  movwf	 ButtonSelected			; No buttons pressed.
  movwf  VideoSelected			; No video sources detected yet.

  movlw  12						; Want 1000 microseconds (not 1024).
  movwf  TMR0					; Start the timer with initial value.

  movlw  (1 << GIE) + (1 << TMR0IE)
  movwf  INTCON					; Enable Interrupts.

;
;  Sample each of the 4 inputs for 1/4 of a second.  Switch to the highest priority "active" input.
;
MainLoop
  movlw  b'11011001'			; Conversion clock internal.  Set channel for AN3.  Converter module on.
  movwf  ADCON0					
  movlw  0						; Get ready to sample AN3.
  movwf	 HighestSample			; Clear highest sample.
  movwf  QuarterFlag			; Reset the quarter flag.
  movlw  VIDEO_4_OFF			; Establish a sample thresholds.
  movwf  VideoOff
  movlw  VIDEO_4_ON			
  movwf  VideoOn
  movlw  b'00001000'			; Bit select if AN3 active.
  movwf  VideoSample
  movlw	 250					; Want a full 1/4 second for sampling.
  movwf	 CountToQuarter
  call   SampleInput			; Sample AN3 for 1/4 of a second and set bit 2 on if there is a video source.

  movlw  b'11010001'			; Conversion clock internal.  Set channel for AN2.  Converter module on.
  movwf  ADCON0					
  movlw  0						; Get ready to sample AN2.
  movwf	 HighestSample			; Clear highest sample.
  movwf  QuarterFlag			; Reset the quarter flag.
  movlw  VIDEO_3_OFF			; Establish a sample thresholds.
  movwf  VideoOff
  movlw  VIDEO_3_ON			
  movwf  VideoOn
  movlw  b'00000100'			; Bit select if AN2 active.
  movwf  VideoSample
  movlw	 250					; Want a full 1/4 second for sampling.
  movwf	 CountToQuarter
  call   SampleInput			; Sample AN2 for 1/4 of a second and set bit 2 on if there is a video source.

  movlw  b'11001001'			; Conversion clock internal.  Set channel for AN1.  Converter module on.
  movwf  ADCON0					
  movlw  0						; Get ready to sample AN1.
  movwf	 HighestSample			; Clear highest sample.
  movwf  QuarterFlag			; Reset the quarter flag.
  movlw  VIDEO_2_OFF			; Establish a sample thresholds.
  movwf  VideoOff
  movlw  VIDEO_2_ON			
  movwf  VideoOn
  movlw  b'00000010'			; Bit select if AN1 active.
  movwf  VideoSample
  movlw	 250					; Want a full 1/4 second for sampling.
  movwf	 CountToQuarter
  call   SampleInput			; Sample AN1 for 1/4 of a second and set bit 1 on if there is a video source.

  movlw  b'11000001'			; Conversion clock internal.  Set channel for AN0.  Converter module on.
  movwf  ADCON0					
  movlw  0						; Get ready to sample AN0.
  movwf	 HighestSample			; Clear highest sample.
  movwf  QuarterFlag			; Reset the quarter flag.
  movlw  VIDEO_1_OFF			; Establish a sample thresholds.
  movwf  VideoOff
  movlw  VIDEO_1_ON			
  movwf  VideoOn
  movlw  b'00000001'			; Bit select if AN0 active.
  movwf  VideoSample
  movlw	 250					; Want a full 1/4 second for sampling.
  movwf	 CountToQuarter
  call   SampleInput			; Sample AN0 for 1/4 of a second and set bit 0 on if there is a video source.

  goto	 MainLoop

;
;  Sample the selected ADC input repeatedly for 1/4 of a second.  When done, see if the "highest"
;  recorded sample value is greater than the "black" threshold (about .4 volts).  If greater, the
;  selected input is considered "active".  Set/clear a bit in the VideoInput accordingly. 
;
SampleInput
  call   CheckButtons			; Make sure we don't miss any button presses. 
  call   NextSample				; Perform a single sample of the selected input.
  movf	 QuarterFlag, f			; Check the status of the quarter flag.
  btfsc	 STATUS, Z				; It it zero?
  goto	 SampleInput			;  Yes - Keep waiting.

CheckVideoOff					;   No - Check to see if the screen is "blank".
  movf	 VideoOff, w			; Set the threshold for a blank screen (different for each input).
  subwf  HighestSample, w		; Subtract threshold from current highest sample.
  btfsc  STATUS, C				; Is threshold higher?
  goto   CheckVideoOn			;   No - Check for video on.
  comf   VideoSample, w 		;  Yes - Video source not detected.
  andwf  VideoInput, f			; Clear the video source bit in the video input.
  goto   SampleInputDone
	
CheckVideoOn
  movf	 VideoOn, w				; Set the threshold for a screen that is on (different for each input).
  subwf  HighestSample, w		; Subtract threshold from current highest sample.
  btfsc  STATUS, C				; Is threshold higher?
  goto	 SetVideoOn				;   No - Video source present.
  goto   SampleInputDone		;  Yes - Don't change the video inputs.

SetVideoOn					
  movf	 VideoSample, w			
  iorwf  VideoInput, f			; Set the video source bit in the video input.

SampleInputDone
  call	 VideoSwitch			; Switch to a new input source (if there has been a change).
  return

;
;  	Do one sample of the selected ADC input channel.  Keep track of the largest sample found so far.
;
NextSample						; Sample the composite-in signal.
  bsf	 ADCON0, GO_DONE		; Initiate an A/D conversion.

WaitForConversion
  btfsc  ADCON0, NOT_DONE		; Wait for conversion to complete.
  goto	 WaitForConversion

  movf	 ADRESH, w				; Get the conversion result (8-bits good enough).
  movwf	 Temp					; Keep in case it's the highest.
  subwf  HighestSample, w		; Subtract w from current highest sample.
  btfsc  STATUS, C				; Is w higher?
  goto   NextSampleDone			;   No - Done with this sample.
  movf	 Temp, w				;  Yes - Set the new highest sample
  movwf  HighestSample

NextSampleDone
  return
 
;
;  See if an override button was pressed.  If a button is pressed, see if the video input
;  needs to be selected or deselected.
;
CheckButtons
  movf	 PORTB, w				
  movwf  Temp		
  movlw  b'00001111'
  andwf	 Temp, w				; Get the inputs.
  btfsc	 STATUS, Z				; Is there a button pressed?
  goto	 DoneButtons			;   No - Done check.
  movwf  ButtonInput			;  Yes - Remember what button was pressed.
  call   ButtonSwitch			; Check for change in selected A/V input.

  movlw  50						; Debounce the button press.
  movwf  DebounceCount			; Set for 1/20 of a second.
DebounceButtonDown
  movf	 DebounceCount, w		; Check the status of the debounce count.
  btfss	 STATUS, Z				; It it zero?
  goto	 DebounceButtonDown		;   No - Keep waiting.

WaitForButtonUp
  movf   PORTB, w
  movwf  Temp
  movlw  b'00001111'
  andwf	 Temp, w				; Get the inputs.
  btfss	 STATUS, Z				; Is there a button pressed?
  goto	 WaitForButtonUp		;  Yes - Keep waiting.

  movlw	 50						; Debounce the button release.
  movwf  DebounceCount			; Set for 1/20 of a second.
DebounceButtonUp
  movf	 DebounceCount, w		; Check the status of the debounce count.
  btfss	 STATUS, Z				; It it zero?
  goto	 DebounceButtonUp		;   No - Keep waiting.

DoneButtons
  return

;
;  Based on the button pressed switch to the appropriate A/V inputs.  Only get called if there is
;  actually a button pressed.
;
ButtonSwitch
  movf	 ButtonInput, w			; Get the override buttons input.
  subwf  ButtonSelected, w		; Compare to see if it matches a button already selected.
  btfss  STATUS, Z				; Was that button already selected?
  goto   SwitchBasedOnButton	;   No - Since we have a different button, switch based on the new button.
  bsf	 LED_5 					;  Yes - Turn off the override light.
  movlw	 0						
  movwf  ButtonSelected			; Clear the button selected.
  movwf  VideoSelected			; Clear video selected so new video source will be picked.
  goto   VideoSwitch			; Since override is no longer in effect, switch based on video inputs.

SwitchBasedOnButton				; Check to see what button was pressed starting with the highest priority.
  bcf	 LED_5  				; Turn on the override light.	
  movlw	 0						; Clear the button selected.
  movwf  ButtonSelected		

CheckButton4	
  btfss	 ButtonInput, 3			; Was button 4 pressed?
  goto	 CheckButton3			;   No - Check button 3.
  bsf	 ButtonSelected, 3		;  Yes - Remember that button 4 was selected.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_4					; Turn on the LED for input 4.
  call   SwitchToInput4			; Switch to input 4.
  goto   DoneSwitch

CheckButton3
  btfss	 ButtonInput, 2			; Was button 3 pressed?
  goto	 CheckButton2			;   No - Check button 2.
  bsf	 ButtonSelected, 2		;  Yes - Remember that button 3 was selected.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_3					; Turn on the LED for input 3.
  call   SwitchToInput3			; Switch to input 3.
  goto   DoneSwitch

CheckButton2
  btfss	 ButtonInput, 1			; Was button 2 pressed?
  goto	 CheckButton1			;   No - Check button 1.
  bsf	 ButtonSelected, 1		;  Yes - Remember that button 2 was selected.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_2					; Turn on the LED for input 2.
  call   SwitchToInput2			; Switch to input 2.
  goto   DoneSwitch

CheckButton1
  btfss	 ButtonInput, 0			; Was button 1 pressed?
  goto   DoneSwitch				;   No - Just update the leds.
  bsf	 ButtonSelected, 0		;  Yes - Remember that button 1 was selected.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_1					; Turn on the LED for input 1.
  call   SwitchToInput1			; Switch to input 1.
  goto   DoneSwitch	

;
;  If there is no override button set, and there is a change to the video settings, switch
;  to the highest priority active video source. 
;
VideoSwitch
  movf	 ButtonSelected, w 
  btfss  STATUS, Z				; Is a button already selected?
  goto   DoneSwitch				;  Yes - If there is a button selected ignore video sources.
  movf   VideoSelected, w		;   No - See if there is a change in the video inputs.
  subwf  VideoInput, w
  btfsc	 STATUS, Z				; Have the video inputs changed?
  goto   DoneSwitch				;   No - Nothing to do.
  movf	 VideoInput, w		    ;  Yes - Remember the new video inputs.
  movwf  VideoSelected
;
;  Select the highest priority video source.
;
CheckVideo4	
  btfss	 VideoSelected, 3		; Was video input 4 active?
  goto	 CheckVideo3			;   No - Check video 3.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_4					;  Yes - Turn on the LED for input 4.
  call   SwitchToInput4			; Switch to input 4.
  goto   DoneSwitch

CheckVideo3	
  btfss	 VideoSelected, 2		; Was video input 3 active?
  goto	 CheckVideo2			;   No - Check video 2.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_3					;  Yes - Turn on the LED for input 3.
  call   SwitchToInput3			; Switch to input 3.
  goto   DoneSwitch

CheckVideo2	
  btfss	 VideoSelected, 1		; Was video input 2 active?
  goto	 CheckVideo1			;   No - Check video 1.
  call   ClearSourceLeds		; Turn off all the video source Leds.
  bcf	 LED_2					;  Yes - Turn on the LED for input 2.
  call   SwitchToInput2			; Switch to input 2.
  goto   DoneSwitch	

CheckVideo1	
  call   ClearSourceLeds		; Turn off all the video source Leds.
  btfsc	 VideoSelected, 0		; Was video input 1 active?
  bcf	 LED_1					;  Yes - Turn on the LED for input 1.
  call   SwitchToInput1			; Switch to input 1 (either way).
 
DoneSwitch
  return

; 
;  Turn off all the video source selected LEDs.
;
ClearSourceLeds
  movlw	 b'11110000'		    ; Clear all the Led outputs
  iorwf  PORTB, f
  return

;
;  Select video source 1.
;
SwitchToInput1
  movf	 PORTA, w
  movwf	 Temp
  bcf	 Temp, A0				; A0-0 A1-0
  nop
  bcf	 Temp, A1
  movf   Temp, w
  movwf  PORTA
  return						

;
;  Select video source 2.
;
SwitchToInput2
  movf	 PORTA, w
  movwf	 Temp
  bsf	 Temp, A0				; A0-1 A1-0
  nop
  bcf	 Temp, A1
  movf   Temp, w
  movwf  PORTA
  return

;
;  Select video source 3.
;
SwitchToInput3
  movf	 PORTA, w
  movwf	 Temp
  bcf	 Temp, A0				; A0-0 A1-1
  nop
  bsf	 Temp, A1
  movf   Temp, w
  movwf  PORTA
  return

;
;  Select video source 4.
;
SwitchToInput4
  movf	 PORTA, w
  movwf	 Temp
  bsf	 Temp, A0				; A0-1 A1-1
  nop
  bsf	 Temp, A1
  movf   Temp, w
  movwf  PORTA
  return

 end
